package pfc.prototipo.jxta;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import net.jxta.id.IDFactory;
import net.jxta.peer.PeerID;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.pipe.PipeID;

/**
 * <p>Encargada de generar los identificador nicos para los distintos Peer</p>
 * <p>Para generar el identificador nico se necesita el nombre del peer, nombre del grupo y string establecido</p>
 * <p>Mtodos obtenidos del manual de referencia de programacin en JXTA</p> 
 * @author Angel
 *
 */

public class PFC_JxtaID 
{
	private static final String SEED = "IDPFC_Network";
	/**
	* Returns a SHA1 hash of string.
	*
	* @param expression to hash
	* @return a SHA1 hash of string or {@code null} if the expression could
	* not be hashed.
	*/
	private static byte[] hash(final String expression) 
	{
		byte[] result;
		MessageDigest digest;
		if (expression == null) 
		{
			throw new IllegalArgumentException("Invalid null expression");
		}

		try 
		{
			digest = MessageDigest.getInstance("SHA1");
		} catch (NoSuchAlgorithmException failed) 
		{
			failed.printStackTrace(System.err);
			RuntimeException failure = new IllegalStateException("Could not get SHA-1 Message");
			failure.initCause(failed);
			throw failure;
		}
		try 
		{		
			byte[] expressionBytes = expression.getBytes("UTF-8");
			result = digest.digest(expressionBytes);
		} catch (UnsupportedEncodingException impossible) 
		{
			RuntimeException failure =
			new IllegalStateException("Could not encode expression as UTF8");
			failure.initCause(impossible);
			throw failure;
		}
		return result;
	}//hash
	

	/**
	* Given a peer name generates a Peer ID who's value is chosen based upon that name.
	*
	* @param peerName instance name
	* @param pgID the group ID encoding
	* @return The PeerID value
	*/
	public static PeerID createPeerID(PeerGroupID pgID, String peerName) 
	{
		String seed = peerName + SEED;
		return IDFactory.newPeerID(pgID, hash(seed.toLowerCase()));
	}

	/**
	* Given a group name generates a Peer Group ID who's value is chosen based
	* upon that name.
	*
	* @param groupName group name encoding value
	* @return The PeerGroupID value
	*/
	public static PeerGroupID createPeerGroupID(final String groupName) 
	{
		//Use lower case to avoid any locale conversion inconsistencies
		return IDFactory.newPeerGroupID(PeerGroupID.defaultNetPeerGroupID,
		hash(SEED + groupName.toLowerCase()));
	}
	/**
	* Contructs and returns an string encoded Infrastructure PeerGroupID.
	*
	* @param groupName the string encoding
	* @return The infraPeerGroupID PeerGroupID
	*/
	public static PeerGroupID createInfraPeerGroupID(String groupName) 
	{
		return createPeerGroupID(groupName);
	}
	
	/**
	 * <p>Crea identificador de Peer a partir del nombre del grupo</p>
	 * @param psNomPeer Nombre del Peer
	 * @param psNomGrupo Nombre del Grupo
	 * @return
	 */
	public static PeerID createPeerID (String psNomPeer, String psNomGrupo)
	{
		return createPeerID(createInfraPeerGroupID(psNomGrupo), psNomPeer);
	}
	
	/**
	* <p>Crea un PipeId a partir del ID Grupo y del nombre del Grupo</p>
	*
	* @param pipeName instance name
	* @param pgID the group ID encoding
	* @return The pipeID value
	*/
	public static PipeID createPipeID(PeerGroupID pgID, String pipeName) 
	{
		String seed = pipeName + SEED;
		return IDFactory.newPipeID(pgID, hash(seed.toLowerCase()));
	}//createPipeID
	
	/**
	 * <p>Crea identificador de Peer a partir del nombre del grupo</p>
	 * @param psNomPipe Nombre del Pipe
	 * @param psNomGrupo Nombre del Grupo
	 * @return
	 */
	public static PipeID createPipeID (String psNomPipe, String psNomGrupo)
	{
		return createPipeID(createInfraPeerGroupID(psNomGrupo), psNomPipe);
	}
	
	
	/**
	* Main method
	*
	* @param args command line arguments. None defined
	*/
	public static void main(String args[]) 
	{
		PeerGroupID infra = createInfraPeerGroupID("PFC_GrupoTrabajo");
		PeerID peerID = createPeerID(infra, "PFC_Nodo1");
		
	}
}
