package pfc.prototipo.jxta;

import java.io.Serializable;
import java.sql.Date;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;

import pfc.prototipo.jxta.superpeer.PFC_SPGestorComunicacionMC;

/**
 * <p>Encapsula cualquier mensaje enviado a los peers; tanto SuperPeer como Peer</p>
 * <p>La tipologia de mensajes es:</p>
 * <p>NUEVO: Cuando un peer crea un nuevo documento e informa al resto que lo ha creado</p>
 * <p>CAMBIO: Cuando un peer realiza cambios en un documento e informa al resto que los ha realizado</p>
 * <p>PETICION_ACTUALIZACION: Cuando un peer ha estado desconectado y pide al SuperPeer que le enve las actualizaciones que haya podido haber</p>
 * <p>ACTUALIZACION: Cuando un peer/superpeer envia un conjunto de actualizaciones realizadas. O bien porque un peer se conecta e informa al resto de todos los cambios realizados; o bien porque un SuperPeer informa a un peer de los cambios que se 
 * han hecho mientras ha estado desconectado</p>
 * @author Angel
 *
 */
public class PFC_Mensaje implements Serializable
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_Mensaje.class);

	//Tipo de mensaje Nuevo documento
	public static String TIPO_NUEVO = "NUEVO";	
	//Tipo de mensaje Baja en un documento
	public static String TIPO_BAJA = "BAJA";
	//Tipo de mensaje Cambios en un documento
	public static String TIPO_CAMBIO = "CAMBIO";
	//Tipo de mensaje Peticion de actualizacion. La realiza un Peer al conectar y es mandada al SuperPeer
	public static String TIPO_PETICION_ACTUALIZACION = "PETICION_ACTUALIZACION";
	//Tipo de mensaje Actualizacion. Es una respuesta del SuperPeer al Peer que le ha pedido PETICION_ACTUALIZACION
	public static String TIPO_ACTUALIZACION = "ACTUALIZACION";
	//Tipo de mensaje donde el SuperPeer avisa al peer peticionario que tiene una alta pendiente
	public static String TIPO_AVISO_ALTA = "AVISO_ALTA";
	//Tipo de mensaje donde el un Peer pide a otro que le envie el alta de un determinado documento. Viene determinado porque SuperPeer avisa
	//a un Peer que existe un documento nuevo en un determinado Peer
	public static String TIPO_PETICION_ALTA = "PETICION_ALTA";
	//Tipo de mensaje donde un Peer envia un documento a otro Peer provocado por la peticion de alta
	public static String TIPO_RESPUESTA_ALTA = "RESPUESTA_ALTA";
	//Tipo de mensaje donde un Peer envia al grupo las actualizaciones realizadas mientras ha estado desconectado
	public static String TIPO_ENVIO_ACTUALIZACIONES = "ENVIO_ACTUALIZACIONES";
	
	
	// Identificador del peer que envia el mensaje
	private String msPeerOrigen = null;
	// Tipo de Mensaje
	private String msTipoMensaje = null;	
	// Nombre del documento con el que se trabaja
	private String msNombreDocumento = null;
	// Propietario del documento
	private String msPropietarioDocumento = null;	
	// id del documento con el que se trabaja
	private String msIdDocumento = null;
	// Fecha documento
	private Date moFechaDocumento = null;	
	// contenido del documento (cuando es nuevo)
	private byte[] mvoContenido = null;
	
	// Lista de cambios realizados en un mismo documento. Es un vector que contiene, String[] donde: 
	// String[0]: path del nodo; String[1]: A, M, B indicando si es una Alta, Baja o Modificacion de Nodo; String[2]: Contenido del nodo a dar de Alta o Modificar,
	//en caso de Baja vaco; String[3]: Orden de las modificaciones
	private Vector moCambio = null;
	
	// Lista de documentos que un determinado Peer tiene censados. La key es el iddocumento, el value un String[3], donde
	// String[0]=propietario,String[1]=nombre,String[2]=fecha
	private Hashtable moDocumentos = null;

	// Vector de PFC_Mensaje donde se indican los documentos que un nodo dispone o donde el SuperPeer indica los cambios a realizar
	// en los documentos
	private Vector moActualizaciones = null;
	

	/**
	 * <p>Se crea un nuevo mensaje estableciendo el Peer que lo crea y el tipo de mensaje</p>
	 * @param psPeer Identificador del peer que crea el mensaje
	 * @param psTipoMensaje Tipo de Mensaje
	 */
	public PFC_Mensaje(String psPeer, String psTipoMensaje)
	{
		this.msPeerOrigen = psPeer;
		this.msTipoMensaje = psTipoMensaje;		
	}//PFC_Mensaje
	
	public PFC_Mensaje( String psTipoMensaje)
	{
		this.msTipoMensaje = psTipoMensaje;
	}//PFC_Mensaje
	
	public Vector getCambio() {
		return moCambio;
	}

	public void setCambio(Vector moCambio) {
		this.moCambio = moCambio;
	}

	public String getPeerOrigen() 
	{
		return msPeerOrigen;
	}
	public void setPeerOrigen(String msPeerOrigen) {
		this.msPeerOrigen = msPeerOrigen;
	}

	public String getTipoMensaje() 
	{
		return msTipoMensaje;
	}

	public String getNombreDocumento() 
	{
		return msNombreDocumento;
	}

	public void setNombreDocumento(String msNombreDocumento) 
	{
		this.msNombreDocumento = msNombreDocumento;
	}

	public String getIdDocumento() 
	{
		return msIdDocumento;
	}

	public void setIdDocumento(String msIdDocumento) 
	{
		this.msIdDocumento = msIdDocumento;
	}

	public String getPropietario() 
	{
		return msPropietarioDocumento;
	}
	public void setPropietario(String msPropietario) {
		this.msPropietarioDocumento = msPropietario;
	}

	public byte[] getContenido() 
	{
		return mvoContenido;
	}

	public void setContenido(byte[] mvoContenido) 
	{
		this.mvoContenido = mvoContenido;
	}
	

	public Vector getActualizaciones() {
		return moActualizaciones;
	}

	public void setActualizaciones(Vector moActualizaciones) {
		this.moActualizaciones = moActualizaciones;
	}
	public Hashtable getDocumentos() {
		return moDocumentos;
	}

	public void setDocumentos(Hashtable moDocumentos) {
		this.moDocumentos = moDocumentos;
	}
	
	public void addActualizacion(PFC_Mensaje poMensaje)
	{
		if (moActualizaciones == null)
			moActualizaciones = new Vector();
		
		moActualizaciones.add(poMensaje);
		
	}

	public Date getFechaDocumento() {
		return moFechaDocumento;
	}
	public void setFechaDocumento(Date poFecha)
	{
		this.moFechaDocumento = poFecha;
	}
}
